package org.openxma.dsl.generator.helper;

import org.openxma.dsl.core.model.DataTypeAndTypeParameter;
import org.openxma.dsl.core.model.EnumType;
import org.openxma.dsl.core.model.SimpleType;
import org.openxma.dsl.core.model.Validator;
import org.openxma.dsl.core.model.ValidatorReference;
import org.openxma.dsl.dom.model.Attribute;
import org.openxma.dsl.generator.helper.style.BasicDataType;
import org.openxma.dsl.pom.model.CheckBox;
import org.openxma.dsl.pom.model.Combo;
import org.openxma.dsl.pom.model.ControlType;
import org.openxma.dsl.pom.model.CustomizeableField;
import org.openxma.dsl.pom.model.GuiElement;
import org.openxma.dsl.pom.model.ListBox;
import org.openxma.dsl.pom.model.Table;
import org.openxma.dsl.pom.model.Text;

public class BasicTypeHelper {
    
    public static BasicType determineBasicType(Attribute attr) {
        String validatorName = BasicTypeHelper.determineRootValidatorName(attr);
        if (ValidatorReference.STRING_VALIDATOR.equals(validatorName)) {
            return BasicType.STRING;
        }
        if (ValidatorReference.NUMBER_VALIDATOR.equals(validatorName)) {
            return BasicType.NUMBER;
        }
        if (ValidatorReference.BOOLEAN_VALIDATOR.equals(validatorName)) {
            return BasicType.BOOLEAN;
        }
        if (ValidatorReference.DATE_VALIDATOR.equals(validatorName)) {
            return BasicType.DATE;
        }
        if (ValidatorReference.TIME_STAMP_VALIDATOR.equals(validatorName)) {
            return BasicType.TIME_STAMP;
        }
        if (ValidatorReference.CUSTOM_VALIDATOR.equals(validatorName)) {
            return BasicType.CUSTOM;
        }
        if (ValidatorReference.I18N_ENUMERATION_VALIDATOR.equals(validatorName)) {
            return BasicType.I18N_ENUMERATION;
        }
        return null;
    }   
    
    public static WidgetType determineWidgetType(GuiElement guiElement) {
        if (guiElement instanceof CustomizeableField) {
            return determineWidgetType((CustomizeableField)guiElement);
        } else if (guiElement instanceof Text) {
            return WidgetType.TEXT;
        } else if (guiElement instanceof CheckBox) {
            return WidgetType.CHECKBOX;            
        } else if (guiElement instanceof Combo) {
            return WidgetType.SIMPLECOMBO; 
        } else if (guiElement instanceof ListBox) {
            return WidgetType.LISTBOX;                
        } else if (guiElement instanceof Table) {
            return WidgetType.TABLE;            
        }
        throw new RuntimeException("Can not determine widget type of: "+guiElement);
    }    
    
    public static WidgetType determineWidgetType(CustomizeableField field) {
        /* Domain formatter overrules all */
        if (field.getFormat() != null) {
            String baseValidatorNameOfField = getBaseValidatorName(field.getFormat());
            if ("I18NEnumerationValidator".equals(baseValidatorNameOfField)) {
                return WidgetType.DOMAINCOMBO;
            }
        }
        /* Explicit control-type-specification of field */
        if (field.getControlType() != null) {
            if (useStaticDataLabel(field)) {
                return WidgetType.DATALABEL;
            } else if (field.getControlType().equals(ControlType.COMBO)) {
                return WidgetType.SIMPLECOMBO;
            } else if (field.getControlType().equals(ControlType.LISTBOX)) {
                return WidgetType.LISTBOX;
            } else if (field.getControlType().equals(ControlType.TEXT)) {
                return WidgetType.TEXT;
            }
        }
        /* Determine from attribute type */
        Attribute attr = field.getAttribute();
        DataTypeAndTypeParameter type = attr.getType();        
        if (isI18NEnumeration(type)) {
            return WidgetType.DOMAINCOMBO;
        }
        if (isBoolean(type)) {
            return WidgetType.CHECKBOX;
        }
        if (isDate(type)) {
            return WidgetType.DATEPICKER;
        }
        return WidgetType.TEXT;        
    }
    
    public static BasicDataType determineBasicDataType(CustomizeableField field) {
        Attribute attr = field.getAttribute();
        DataTypeAndTypeParameter type = attr.getType();
        if (type.getDataType() instanceof SimpleType) {
            SimpleType simpleType = (SimpleType)type.getDataType();
            String typeName = NameExtension.getFullyQualifiedName(simpleType,true);       
            // TODO change this dummy implementation to real type determination
            if ("java.math.BigDecimal".equals(typeName)) {
                typeName="BigDecimal";
            } else if ("java.util.Date".equals(typeName)) {
                typeName="Date";
            }
            return BasicDataType.valueOf(typeName);                   
        }
        return null;        
    }
    
    public static String determineRootValidatorName(Attribute attr) {
        ValidatorReference rootValidatorReference = determineRootValidator(attr);
        if (rootValidatorReference == null) {
            if (attr.getType().getDataType() instanceof EnumType) {
                return ValidatorReference.I18N_ENUMERATION_VALIDATOR;
            }
            throw new RuntimeException("Root validator is not defined for '"+attr.getType().getDataType().getName()+"'");
        }
        return determineRootValidatorName(rootValidatorReference);                
    }

    public static ValidatorReference determineRootValidator(Attribute attr) {
        ValidatorReference validatorReference = attr.getType().getValidatorReferenceDefinition();
        if (validatorReference == null) {
            return null;
        }
        ValidatorReference rootValidatorReference = validatorReference.getRootValidatorReference();
        return rootValidatorReference;
    }
    
    public static String determineRootValidatorName(ValidatorReference rootValidatorReference) {
        String validatorName = rootValidatorReference.getValidator().getName();
        return validatorName;                
    }
    
    public static boolean useStaticDataLabel(CustomizeableField cf) {
        if (cf.getControlType() != null) {
            return cf.getControlType().equals(ControlType.LABEL);
        }
        return false;
    }
    
    public static boolean isI18NEnumeration(DataTypeAndTypeParameter dataTypeAndTypeParameter) {
        String validatorName = getBaseValidatorName(dataTypeAndTypeParameter);
        return "I18NEnumerationValidator".equals(validatorName);
    }

    public static boolean isBoolean(DataTypeAndTypeParameter dataTypeAndTypeParameter) {
        String validatorName = getValidatorName(dataTypeAndTypeParameter);
        return "BooleanValidator".equals(validatorName);
    }

    public static boolean isDate(DataTypeAndTypeParameter dataTypeAndTypeParameter) {
        String validatorName = getValidatorName(dataTypeAndTypeParameter);
        return "DateValidator".equals(validatorName);
    }
    
    public static String getBaseValidatorName(ValidatorReference validatorReference) {
        Validator val = validatorReference.getValidator();
        ValidatorReference superValRef = val.getSuperValidator();
        if (superValRef != null) {
            return getBaseValidatorName(superValRef);
        }
        return validatorReference.getValidator().getName();
    }

    public static String getBaseValidatorName(DataTypeAndTypeParameter dataTypeAndTypeParameter) {
        if (dataTypeAndTypeParameter.getDataType() instanceof EnumType) {
            return "I18NEnumerationValidator";
        }
        ValidatorReference validatorReference = SimpleTypeExtension.getValidatorReferenceDefinition(dataTypeAndTypeParameter);
        return getBaseValidatorName(validatorReference);
    }

    public static String getValidatorName(DataTypeAndTypeParameter dataTypeAndTypeParameter) {
        ValidatorReference validatorReference = SimpleTypeExtension.getValidatorReferenceDefinition(dataTypeAndTypeParameter);
        if (validatorReference != null) {
            return validatorReference.getValidator().getName();
        } else if (dataTypeAndTypeParameter.getDataType() instanceof EnumType) {
            return "I18NEnumerationValidator";
        } else {
            return null;
        }
    }
}
