package org.openxma.dsl.generator.helper;

import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.openxma.dsl.core.model.EqualityExpr;
import org.openxma.dsl.pom.model.CollapsedFlag;
import org.openxma.dsl.pom.model.Composite;
import org.openxma.dsl.pom.model.EditableFlag;
import org.openxma.dsl.pom.model.EnabledFlag;
import org.openxma.dsl.pom.model.FieldFlag;
import org.openxma.dsl.pom.model.GuiElement;
import org.openxma.dsl.pom.model.IElementWithLayoutData;
import org.openxma.dsl.pom.model.MandatoryFlag;
import org.openxma.dsl.pom.model.ReferencedXMAComposite;
import org.openxma.dsl.pom.model.ReferencedXMAPage;
import org.openxma.dsl.pom.model.SetOfGuiElements;
import org.openxma.dsl.pom.model.TabFolder;
import org.openxma.dsl.pom.model.TabPage;
import org.openxma.dsl.pom.model.VisibleFlag;
import org.openxma.dsl.pom.model.XmadslComposite;

import at.spardat.xma.guidesign.XMAComposite;
import at.spardat.xma.guidesign.XMAWidget;
import at.spardat.xma.guidesign.flex.CollapseChain;
import at.spardat.xma.guidesign.flex.CollapseGroup;

/**
 * This class allows to deal with fieldFlags of a List more efficient.
 *
 */
public class FieldFlags {
	
	private List<FieldFlag> sourceFieldFlags;
	
	private EditableFlag editableFlag;
	private EnabledFlag enabledFlag;
	private VisibleFlag visibleFlag;
	private MandatoryFlag mandatoryFlag;
	private CollapsedFlag collapsedFlag;
	
	/**
	 * Get the list of FieldFlags of a EObject
	 * 
	 * @param obj
	 * @return
	 */
	public static List<FieldFlag> getFieldFlagsOfContainer(EObject obj) {
		if (obj instanceof Composite == false && obj instanceof TabFolder == false) {
			return null;
		}
		if (obj instanceof ReferencedXMAPage) {
			return ((ReferencedXMAPage)obj).getFieldFlags();
		} else if (obj instanceof ReferencedXMAComposite) {
			return ((ReferencedXMAComposite)obj).getFieldFlags();
		} else if (obj instanceof TabPage) {
			return ((TabPage)obj).getFieldFlags();
		}  else if (obj instanceof TabFolder) {
			return ((TabFolder)obj).getFieldFlags();
		}  else if (obj instanceof XmadslComposite) {
			return ((XmadslComposite)obj).getFieldFlags();
		}
		return null;
	}
	
	/**
	 * 
	 * @param obj
	 * @return
	 */
	public static List<FieldFlag> getFieldFlagsOfAllGuiElements(EObject obj) {
		
		if (obj instanceof IElementWithLayoutData) {
			return ((IElementWithLayoutData)obj).getFieldFlags();
		}
		return getFieldFlagsOfContainer(obj);
	}	
	
	/**
	 * Constructor
	 * 
	 * @param fieldFlags
	 */
	public FieldFlags(List<FieldFlag> fieldFlags) {
		sourceFieldFlags = fieldFlags;		
		for (FieldFlag flag : fieldFlags) {
			if (flag instanceof EnabledFlag) {
				enabledFlag = (EnabledFlag)flag;
				continue;
			} else if (flag instanceof EditableFlag) {
				editableFlag = (EditableFlag)flag;
				continue;
			} else if (flag instanceof VisibleFlag) {
				visibleFlag = (VisibleFlag)flag;
				continue;
			} else if (flag instanceof MandatoryFlag) {
				mandatoryFlag = (MandatoryFlag)flag;
				continue;
			} else if (flag instanceof CollapsedFlag) {
				collapsedFlag = (CollapsedFlag)flag;
				continue;
			}
			throw new RuntimeException("Class of fieldFlag is not supported: "+flag);
		}				
	}
	
	public void updateSourceFieldFlagList() {
		updateFieldFlagList(sourceFieldFlags);
	}
	
	/**
	 * Update the list of FieldFlags
	 * 
	 * @param fieldFlags
	 */
	public void updateFieldFlagList(List<FieldFlag> fieldFlags) {		
		fieldFlags.clear();
		if (enabledFlag != null) fieldFlags.add(enabledFlag);
		if (editableFlag != null) fieldFlags.add(editableFlag);
		if (visibleFlag != null) fieldFlags.add(visibleFlag);
		if (mandatoryFlag != null) fieldFlags.add(mandatoryFlag);
		if (collapsedFlag != null) fieldFlags.add(collapsedFlag);		
	}
	
	/**
	 * 
	 * @param obj
	 * @param parentFieldFlagList
	 */
	public static void joinFieldFlags(EObject obj, List<FieldFlag> parentFieldFlagList) {
		List<FieldFlag>childFieldFlagList = FieldFlags.getFieldFlagsOfAllGuiElements(obj);		
		if (childFieldFlagList != null) { // Is never null in case the fieldFlags-property exists
			FieldFlags parentFieldFlags = new FieldFlags(parentFieldFlagList);
			FieldFlags childFieldFlags = new FieldFlags(childFieldFlagList);			
			// Joining
			childFieldFlags.joinEditableFlag(parentFieldFlags);
			childFieldFlags.joinEnabledFlag(parentFieldFlags);
			childFieldFlags.joinMandatoryFlag(parentFieldFlags);					
			childFieldFlags.joinVisibleFlag(parentFieldFlags);			
			// Update list of FieldFlags 
			childFieldFlags.updateFieldFlagList(childFieldFlagList);
		}		
	}	
	
	/**
	 * 
	 * @param fieldFlagsToJoin
	 */
	public void joinEditableFlag(FieldFlags fieldFlagsToJoin) {
		if (fieldFlagsToJoin != null && fieldFlagsToJoin.editableFlag != null) {
			if (editableFlag == null) {
				editableFlag = (EditableFlag)EcoreUtil.copy(fieldFlagsToJoin.editableFlag);
			} else {
				EqualityExpr parentExpression = (EqualityExpr)EcoreUtil.copy(fieldFlagsToJoin.editableFlag.getExpression());				
				EqualityExpr childExpression = editableFlag.getExpression();					
				EqualityExpr joinedExpression = XmadslExpression.and(childExpression,parentExpression);					
				editableFlag.setExpression(joinedExpression);									
			}			
		}
	}
	
	/**
	 * 
	 * @param fieldFlagsToJoin
	 */
	public void joinEnabledFlag(FieldFlags fieldFlagsToJoin) {
		if (fieldFlagsToJoin != null && fieldFlagsToJoin.enabledFlag != null) {
			if (enabledFlag == null) {
				enabledFlag = (EnabledFlag)EcoreUtil.copy(fieldFlagsToJoin.enabledFlag);
			} else {
				EqualityExpr parentExpression = (EqualityExpr)EcoreUtil.copy(fieldFlagsToJoin.enabledFlag.getExpression());				
				EqualityExpr childExpression = enabledFlag.getExpression();					
				EqualityExpr joinedExpression = XmadslExpression.and(childExpression,parentExpression);					
				enabledFlag.setExpression(joinedExpression);									
			}			
		}
	}
	
	/**
	 * 
	 * @param fieldFlagsToJoin
	 */
	public void joinMandatoryFlag(FieldFlags fieldFlagsToJoin) {
		if (fieldFlagsToJoin != null && fieldFlagsToJoin.mandatoryFlag != null) {
			if (mandatoryFlag == null) {
				mandatoryFlag = (MandatoryFlag)EcoreUtil.copy(fieldFlagsToJoin.mandatoryFlag);
			} else {
				EqualityExpr parentExpression = (EqualityExpr)EcoreUtil.copy(fieldFlagsToJoin.mandatoryFlag.getExpression());				
				EqualityExpr childExpression = mandatoryFlag.getExpression();					
				EqualityExpr joinedExpression = XmadslExpression.or(childExpression,parentExpression);					
				mandatoryFlag.setExpression(joinedExpression);									
			}			
		}
	}	
	
	/**
	 * 
	 * @param fieldFlagsToJoin
	 */
	public void joinVisibleFlag(FieldFlags fieldFlagsToJoin) {
		if (fieldFlagsToJoin != null && fieldFlagsToJoin.visibleFlag != null) {
			if (visibleFlag == null) {
				visibleFlag = (VisibleFlag)EcoreUtil.copy(fieldFlagsToJoin.visibleFlag);
			} else {
				EqualityExpr parentExpression = (EqualityExpr)EcoreUtil.copy(fieldFlagsToJoin.visibleFlag.getExpression());				
				EqualityExpr childExpression = visibleFlag.getExpression();					
				EqualityExpr joinedExpression = XmadslExpression.and(childExpression,parentExpression);					
				visibleFlag.setExpression(joinedExpression);									
			}			
		}
	}
	
	public static CollapseGroup getCollapseGroupForWidget(XMAWidget widget) {
		XMAComposite container = widget.eContainer() instanceof XMAComposite ? ((XMAComposite)widget.eContainer()) : null;
		/* Widget (Composite) which is representing a cell */
        SetOfGuiElements setOfGuiElements = getSetOfGuiElementsForCellComposite(widget);
        if (setOfGuiElements != null) {
            CollapseGroup collapseGroup = getCollapseGroupForWidget(container,widget);
            return collapseGroup;
        } 
        /* Widget within a cell */        
        if (isMainWidgetInCollapseGroup(container,widget)) {
    		XMAComposite xmaComposite = container != null && container.eContainer() instanceof XMAComposite ? (XMAComposite)container.eContainer() : null;
    		if (xmaComposite != null) {
    			CollapseGroup collapseGroup = getCollapseGroupForWidget(xmaComposite,container);		
    			return collapseGroup;
    		} 
        }		
		return null;
	}

	private static CollapseGroup getCollapseGroupForWidget(XMAComposite xmaComposite, XMAWidget widget) {
		EList collapseChains = xmaComposite.getCollapseChains();
		if (collapseChains == null || collapseChains.size()<1) return null;
		CollapseChain collapseChain = (CollapseChain)collapseChains.get(0);
		EList<CollapseGroup> collapseGroups = collapseChain.getCollapseGroups();
		if (collapseGroups == null || collapseGroups.size()<1) return null;
		for (CollapseGroup cg : collapseGroups) {
			if (cg.getMainWidget() == widget) {
				return cg;
			}
		}
		return null;
	}
	
	private static SetOfGuiElements getSetOfGuiElementsForCellComposite(XMAWidget widget) {
        if (widget instanceof XMAComposite) {
            EObject modelSource = WidgetExtension.getSourceModelElementProperty(widget);
            if (modelSource instanceof SetOfGuiElements) {
                return (SetOfGuiElements)modelSource;
            }
        }	    
        return null;
	}
	
	private static boolean isMainWidgetInCollapseGroup(XMAComposite composite, XMAWidget widget) {		
		if (composite == null) return false;
		GuiElement widgetGuiElement = WidgetExtension.getGuiElementProperty(widget);
		Object containerGuiElement = (widgetGuiElement != null ) ? widgetGuiElement.eContainer(): null;		
		if (widgetGuiElement != null && containerGuiElement instanceof SetOfGuiElements) {
			SetOfGuiElements set = (SetOfGuiElements) containerGuiElement;
			int index = set.getElements().indexOf(widgetGuiElement);
			if (index == 0) return true;
		}
		return false;		
	}
	
	public EditableFlag getEditableFlag() {
		return editableFlag;
	}

	public EnabledFlag getEnabledFlag() {
		return enabledFlag;
	}

	public VisibleFlag getVisibleFlag() {
		return visibleFlag;
	}

	public MandatoryFlag getMandatoryFlag() {
		return mandatoryFlag;
	}

	public CollapsedFlag getCollapsedFlag() {
		return collapsedFlag;
	}

	public void setEditableFlag(EditableFlag editableFlag) {
		this.editableFlag = editableFlag;
	}

	public void setEnabledFlag(EnabledFlag enabledFlag) {
		this.enabledFlag = enabledFlag;
	}

	public void setVisibleFlag(VisibleFlag visibleFlag) {
		this.visibleFlag = visibleFlag;
	}

	public void setMandatoryFlag(MandatoryFlag mandatoryFlag) {
		this.mandatoryFlag = mandatoryFlag;
	}

	public void setCollapsedFlag(CollapsedFlag collapsedFlag) {
		this.collapsedFlag = collapsedFlag;
	}
	
	
	

}
