package org.openxma.dsl.generator.impl;

import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.emf.ecore.EObject;
import org.openxma.dsl.core.model.Style;
import org.openxma.dsl.generator.helper.PresentationModelExtension;
import org.openxma.dsl.generator.helper.WidgetExtension;
import org.openxma.dsl.pom.model.GuiElement;
import org.openxma.dsl.pom.model.Page;
import org.openxma.dsl.pom.model.StylePropertyBackgroundColor;
import org.openxma.dsl.pom.model.StylePropertyFont;
import org.openxma.dsl.pom.model.StylePropertyForegroundColor;
import org.openxma.dsl.pom.model.StyleSpecification;
import org.openxma.dsl.pom.model.StyleSpecificationProperty;
import org.openxma.dsl.pom.scoping.PomDslScopeProvider;

import at.spardat.xma.guidesign.XMAPage;
import at.spardat.xma.guidesign.XMAWidget;

public class DefaultWidgetStyling {		
	
	class ColorInfo {
		String name;
		Integer red;
		Integer green;
		Integer blue;

		public String getJavaName() {
			StringBuilder javaName= new StringBuilder("color_");
			if (name != null) {
				javaName.append(name);
			} else {
				javaName.append(red+"_"+green+"_"+blue);
			}
			return javaName.toString();
		}
		
		protected String getRGBParameters() {		
			StringBuilder parameters= new StringBuilder();
			parameters.append(red+", "+green+", "+blue);
			return parameters.toString();
		}	
		
		public String getColorAccessMethod() {
			if (name != null) {
				return "getComposite().getDisplay().getSystemColor(SWT.COLOR_"+name.toUpperCase()+")";
			} else {
				return "getComponent().getColor("+getRGBParameters()+")";
			}
		}
		
	}
	
	class FontInfo {
		Set<String> names = new LinkedHashSet<String>();
		Set<String> styles = new LinkedHashSet<String>();
		Integer size;
		
		public String getJavaName() {
			StringBuilder javaName= new StringBuilder("font");
			for (String name : names) {
				javaName.append(name);
			}
			if (size != null && size>0) javaName.append(size);
			for (String style : styles) {
				javaName.append(style);
			}
			return javaName.toString();
		}
		
		public String getFontParameters(String templateName) {
												
			StringBuilder parameters= new StringBuilder();
			if (names.size() > 0) {
				parameters.append("\""+names.iterator().next()+"\", ");
			} else {
				parameters.append(templateName+".getName(), ");				
			}
			if (size != null && size>0) {
				parameters.append(size+", ");
			} else {
				parameters.append(templateName+".getHeight(), ");
			}
			if (styles.size()>0) {
				boolean first=true;
				for (String style : styles) {
					if (!first) parameters.append("|");
					if (style.equalsIgnoreCase("standard")) {
					    style="NORMAL";
					}
					parameters.append("SWT."+style.toUpperCase());
					first=false;
				}
			} else {
				parameters.append(templateName+".getStyle()");
			}
			return parameters.toString();
		}
	}
	
	
	class LazyStringBuilder {
		private StringBuilder buf;
		private Map<String,FontInfo> fonts;
		
		void append(String s) {
			if (buf == null) {
				buf = new StringBuilder();
			}
			buf.append(s);
		}
		
		
		void putFont(FontInfo font) {
			if (fonts == null) {
				fonts = new LinkedHashMap<String,FontInfo>();
			}
			fonts.put(font.getJavaName(),font);			
		}
		
		boolean hasValue() {
			return buf != null && buf.length()>0;
		}
		
		StringBuilder getBuffer() {
			return buf;
		}
		
		
		Map<String,FontInfo> getFonts() {
			return fonts;
		}		
	}
	
	class SeperatedBuffers {
		

		private Map<String,ColorInfo> colors;		
		LazyStringBuilder foregroundColor = new LazyStringBuilder();
		LazyStringBuilder backgroundColor = new LazyStringBuilder();
		LazyStringBuilder font = new LazyStringBuilder();	
		
		void putColor(ColorInfo colorInfo) {
			if (colors == null) {
				colors = new LinkedHashMap<String,ColorInfo>();
			}
			colors.put(colorInfo.getJavaName(), colorInfo);
		}
		
		Map<String,ColorInfo> getColors() {
			return colors;
		}		
		
		void appendTo(StringBuilder buf) {
			
			
			if (getColors() != null && getColors().size()>0) {
				buf.append("\n        /* Color variable */\n");
				for (ColorInfo colorInfo : getColors().values()) {
					buf.append("        org.eclipse.swt.graphics.Color "+colorInfo.getJavaName()+" = "+colorInfo.getColorAccessMethod()+";\n");
				}
			}					
			
			if (foregroundColor.hasValue()) {
				buf.append("\n        /* Foreground color */\n");
				buf.append(foregroundColor.getBuffer());
			}		
			if (backgroundColor.hasValue()) {
				buf.append("\n        /* Background color */\n");		
				buf.append(backgroundColor.getBuffer());
			}						
			if (font.hasValue()) {
				buf.append("\n        /* Font */\n");
				buf.append("        org.eclipse.swt.graphics.FontData templateFontData = getComposite().getFont().getFontData()[0];\n");				
				for (String fontName : font.getFonts().keySet()) {
					FontInfo fontInfo = font.getFonts().get(fontName);					
					buf.append("        org.eclipse.swt.graphics.Font "+fontName+"=getComponent().getFont("+fontInfo.getFontParameters("templateFontData")+");\n");
				}		
				buf.append(font.getBuffer());				
			}									
		}
	}
	
	protected void genAdditionalCreateWidgetsCodeClient(DomPageLogicAttachmentStrategy pageLogicAttachmentStrategy, XMAPage xmaPage, StringBuilder buf) {
		
		SeperatedBuffers seperatedBuffers = new SeperatedBuffers();		
		
		// Handle pageComposite
		Page dslPage = pageLogicAttachmentStrategy.getDslPage(xmaPage);
		if (dslPage != null) {
			List<Style> pageStyles = PresentationModelExtension.getStyleProperty(dslPage);
		    if (pageStyles != null && pageStyles.size()>0) {
			    applyStyles(seperatedBuffers, xmaPage.getComposite(), pageStyles);
		    }
		}
		
	    // Handle all other content
		List<EObject> xmaElements = PomDslScopeProvider.getNsElementsInRange(xmaPage, null);
		for (EObject xmaObj : xmaElements) {
			if (xmaObj instanceof XMAWidget) {
				XMAWidget xmaWidget = (XMAWidget)xmaObj;
				GuiElement guiElement = WidgetExtension.getGuiElementProperty((XMAWidget)xmaObj);
				if (guiElement != null) {
					if (PresentationModelExtension.canModelElementHaveStyle(guiElement)) {
					    List<Style> objStyles = PresentationModelExtension.getStyleProperty(guiElement);
					    //System.out.println("GuiElement= "+guiElement+", style= "+objStyles);
					    if (objStyles != null && objStyles.size()>0) {
						    applyStyles(seperatedBuffers, xmaWidget, objStyles);
					    }
					}				
				}
			}
		}
		
		seperatedBuffers.appendTo(buf);
	}	
	
	protected void applyStyles(SeperatedBuffers buf, XMAWidget xmaWidget, List<Style> objStyles) {
		for (Style style : objStyles) {
			if (style instanceof StyleSpecification) {
				StyleSpecification styleSpec = (StyleSpecification)style;
				for (StyleSpecificationProperty styleProp : styleSpec.getStyleProperty()) {
                    if (styleProp instanceof StylePropertyBackgroundColor) {
						StylePropertyBackgroundColor sb = (StylePropertyBackgroundColor)styleProp;
						ColorInfo colorInfo = new ColorInfo();						
						if (sb.getColorID() != null) {
							colorInfo.name = sb.getColorID();
						} else {
							colorInfo.red = sb.getRed();
							colorInfo.green = sb.getGreen();							
							colorInfo.blue = sb.getBlue();														
						}
						buf.putColor(colorInfo);						
						buf.backgroundColor.append("        "+xmaWidget.getNamWidget()+"."+"setBackground("+colorInfo.getJavaName()+");\n");						
					} else if (styleProp instanceof StylePropertyForegroundColor) {
						StylePropertyForegroundColor sb = (StylePropertyForegroundColor)styleProp;
						ColorInfo colorInfo = new ColorInfo();						
						if (sb.getColorID() != null) {
							colorInfo.name = sb.getColorID();					
						} else {
							colorInfo.red = sb.getRed();
							colorInfo.green = sb.getGreen();							
							colorInfo.blue = sb.getBlue();														
						}
						buf.putColor(colorInfo);						
						buf.foregroundColor.append("        "+xmaWidget.getNamWidget()+"."+"setForeground("+colorInfo.getJavaName()+");\n");						
					} else if (styleProp instanceof StylePropertyFont) {
						StylePropertyFont fp = (StylePropertyFont)styleProp;
						FontInfo fi = new FontInfo();
						for (String name : fp.getName()) {
							fi.names.add(name);
						}
						if (fp.getSize()>=0) fi.size=fp.getSize();
						for (String fontStyle : fp.getStyles()) {
							fi.styles.add(fontStyle);
						}
						buf.font.putFont(fi);
						buf.font.append("        "+xmaWidget.getNamWidget()+"."+"setFont("+fi.getJavaName()+");\n");						
					}
				}
			}
		}
	}	

}
