package org.openxma.dsl.generator.impl;

import static org.openxma.dsl.generator.GeneratorConstants.INDENT_COUNTER;
import static org.openxma.dsl.generator.GeneratorConstants.OUTLET_NAME_GEN_RESOURCE;
import static org.openxma.dsl.generator.GeneratorConstants.OUTLET_NAME_GEN_TEST;
import static org.openxma.dsl.generator.GeneratorConstants.OUTLET_NAME_RESOURCE;
import static org.openxma.dsl.generator.GeneratorConstants.OUTLET_NAME_SOURCE;
import static org.openxma.dsl.generator.GeneratorConstants.OUTLET_NAME_TEST;

import java.io.File;
import java.util.ArrayList;
import java.util.Map;

import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.plugin.EcorePlugin;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.mwe.core.monitor.ProgressMonitor;
import org.eclipse.xpand2.XpandExecutionContext;
import org.eclipse.xpand2.XpandExecutionContextImpl;
import org.eclipse.xpand2.XpandFacade;
import org.eclipse.xpand2.output.Outlet;
import org.eclipse.xpand2.output.Output;
import org.eclipse.xpand2.output.OutputImpl;
import org.eclipse.xpand2.output.PostProcessor;
import org.eclipse.xtend.expression.Variable;
import org.eclipse.xtend.type.impl.java.JavaBeansMetaModel;
import org.eclipse.xtend.typesystem.emf.EmfRegistryMetaModel;
import org.openxma.dsl.common.extensions.EObjectPropertiesAdapter;
import org.openxma.dsl.core.model.Model;
import org.openxma.dsl.generator.Generator;
import org.openxma.dsl.generator.GeneratorConfiguration;
import org.openxma.dsl.generator.GeneratorConstants;
import org.openxma.dsl.generator.component.ConditionalJavaGeneratorVetoStrategy;
import org.openxma.dsl.generator.component.HbmFileMergePostProcessor;
import org.openxma.dsl.generator.component.JavaImportPostProcessor;
import org.openxma.dsl.generator.output.EclipseResourceOutlet;
import org.openxma.dsl.generator.output.JavaOutlet;

import com.google.common.collect.Lists;
import com.google.inject.internal.Maps;

public class DomGenerator extends AbstractGenerator {
    private XpandExecutionContext xpandExecutionContext;
    private XpandFacade xpandFacade;

    public void init(GeneratorConfiguration generatorConfiguration) {
        super.init(generatorConfiguration);
        Map<String, Variable> globalVars = Maps.newHashMap();
        Output output = createOutput(globalVars);
        xpandExecutionContext = createXpandExecutionContext(output, globalVars);
    }

    public void generate(Resource resource, ProgressMonitor progressMonitor) {
        logger.info("Generator Version: " + getGeneratorVersion());
        if (resource == null) {
            throw new IllegalArgumentException("parameter '" + resource + "' must not be null");
        }
        // totalWork value '10' corresponds to the steps invoked in DomainModel.xpt
        progressMonitor.beginTask("on " + resource.getURI().lastSegment(), 10);
        try {
            modify(progressMonitor, resource);
            progressMonitor.worked(1);
            if (resource.getEObject("/") instanceof Model) {
                ((XpandExecutionContextImpl) xpandExecutionContext).setMonitor(progressMonitor);
                getXpandFacade().evaluate(GeneratorConstants.DOMAIN_MODEL_TEMPLATE_MAIN, resource.getEObject("/"));
            }
        } finally {
            progressMonitor.done();
        }
    }

    private Output createOutput(Map<String, Variable> globalVars) {
        Output output = new OutputImpl();
        JavaImportPostProcessor importPostProcessor = new JavaImportPostProcessor();
        globalVars.put(JavaImportPostProcessor.VAR_NAME, new Variable(JavaImportPostProcessor.VAR_NAME,
                importPostProcessor));

        URI sourceFolder = getConfiguration().getSourceFolder();
        URI resourceFolder = getConfiguration().getResourceFolder();
        URI testSourceFolder = getConfiguration().getTestSourceFolder();
        URI genSourceFolder = getConfiguration().getGenSourceFolder();
        URI genResourceFolder = getConfiguration().getGenResourceFolder();
        URI genTestSourceFolder = getConfiguration().getGenTestSourceFolder();

        Outlet sourceOutlet = createOutlet(OUTLET_NAME_SOURCE, sourceFolder, false, true, importPostProcessor);
        Outlet resourceOutlet = createOutlet(OUTLET_NAME_RESOURCE, resourceFolder, false, true, null);
        Outlet genSourceOutlet = createOutlet(null, genSourceFolder, true, true, importPostProcessor);
        genSourceOutlet.addVetoStrategy(new ConditionalJavaGeneratorVetoStrategy(sourceFolder.toFileString()));
        Outlet genResourceOutlet = createOutlet(OUTLET_NAME_GEN_RESOURCE, genResourceFolder, true, true, null);
        genResourceOutlet.addPostprocessor(new HbmFileMergePostProcessor());

        output.addOutlet(genSourceOutlet);
        output.addOutlet(genResourceOutlet);
        output.addOutlet(sourceOutlet);
        output.addOutlet(resourceOutlet);
        
        if (testSourceFolder!=null) {
            output.addOutlet(createOutlet(OUTLET_NAME_TEST, testSourceFolder, false, false, importPostProcessor));
        }
        if (genTestSourceFolder!=null) {
            output.addOutlet(createOutlet(OUTLET_NAME_GEN_TEST, genTestSourceFolder, true, false, null));
        }
        return output;
    }

    private Outlet createOutlet(String name, URI uri, boolean overwrite, boolean required, PostProcessor postProcessor) {
        Outlet outlet = null;
        if (uri.isPlatformResource()) {
            outlet = createResourceOutlet(name, uri, overwrite, required);
        } else if (uri.isFile()) {
            outlet = createOutlet(name, uri, overwrite, required);
        }
        if (outlet != null && postProcessor != null) {
            outlet.addPostprocessor(postProcessor);
        }
        return outlet;
    }

    private Outlet createOutlet(String name, URI uri, boolean overwrite, boolean required) {
        Outlet outlet = new JavaOutlet(uri.toFileString());
        outlet.setName(name);
        outlet.setOverwrite(overwrite);
        return outlet;
    }

    private Outlet createResourceOutlet(String name, URI uri, boolean overwrite, boolean required) {
        IFolder folder = EcorePlugin.getWorkspaceRoot().getFolder((IPath) toPath(uri));
        if (required && (folder == null || !folder.exists())) {
            String message = "Skip generation because the configured output directory '" + uri + "' doesn't exist.";
            logger.error(message);
            throw new IllegalStateException(message);
        }
        return new EclipseResourceOutlet(name, folder, overwrite);
    }

    private XpandExecutionContextImpl createXpandExecutionContext(Output output, Map<String, Variable> globalVars) {
        XpandExecutionContextImpl xpandExecutionContext = null;
        if (output != null) {
            globalVars.put(INDENT_COUNTER, new Variable(INDENT_COUNTER, 0L));
            xpandExecutionContext = new XpandExecutionContextImpl(output, null, globalVars, null, null);
            xpandExecutionContext.registerMetaModel(new EmfRegistryMetaModel());
            xpandExecutionContext.registerMetaModel(new JavaBeansMetaModel());
            String xpandAdvice = getConfiguration().getProperty(GeneratorConstants.XPAND_ASPECT_FILE, "");
            if (!"".equals(xpandAdvice)) {
                URI xpandAdviceFileURI = getXpandAdviceFile(xpandAdvice);
                if (xpandAdviceFileURI != null) {
                    logger.info("Register xpandAdviceFile '" + xpandAdviceFileURI + "'");
                    xpandExecutionContext.registerAdvices(xpandAdviceFileURI.trimFileExtension().toFileString());
                } else {
                    logger.warn("Configured xpandAdviceFile '" + xpandAdvice + "' not found!");
                }
            }
        }
        return xpandExecutionContext;
    }

    private URI getXpandAdviceFile(String xpandAdvice) {
        URI xpandAdviceFileURI = null;
        if (EcorePlugin.getWorkspaceRoot() != null) {
            xpandAdviceFileURI = getXpandAdviceFileFromWorkspace(xpandAdvice);
        } else {
            File file = new File(xpandAdvice);
            if (file.exists()) {
                xpandAdviceFileURI = URI.createURI(file.toURI().toString());
            }
        }
        return xpandAdviceFileURI;
    }

    private URI getXpandAdviceFileFromWorkspace(String xpandAdvice) {
        URI xpandAdviceFileURI = null;
        IResource resource = EcorePlugin.getWorkspaceRoot().findMember(Path.fromPortableString(xpandAdvice));
        if (resource != null && resource.exists()) {
            xpandAdviceFileURI = URI.createURI(resource.getLocationURI().toString());
        }
        return xpandAdviceFileURI;
    }

    private void modify(ProgressMonitor progressMonitor, Resource modelResource) {
        // copy to avoid CCME due to lazy on-demand loading
        ArrayList<Resource> resources = Lists.newArrayList(modelResource.getResourceSet().getResources());
        for (Resource resource : resources) {
            if (shouldBeModified(resource)) {
                progressMonitor.subTask(" - pre-process model " + resource.getURI().lastSegment());
                logger.info("Pre-process model '" + resource.getURI().lastSegment() + "'");
                getConfiguration().getModelModifier().modifyModel((Model) resource.getEObject("/"));
                EObjectPropertiesAdapter.setProperty(resource, GeneratorConstants.MODIFIED_MODEL, Boolean.TRUE);
            }
        }
    }

    private boolean shouldBeModified(Resource resource) {
        return !hasAlreadyBeenProcessedByModifier(resource) && resource.getEObject("/") instanceof Model;
    }

    private XpandFacade getXpandFacade() {
        if (xpandFacade == null) {
            xpandFacade = XpandFacade.create(xpandExecutionContext);
        }
        return xpandFacade;
    }

    private boolean hasAlreadyBeenProcessedByModifier(Resource resource) {
        return EObjectPropertiesAdapter.hasProperty(resource, GeneratorConstants.MODIFIED_MODEL);
    }

}